#!/usr/bin/python

# !!!!!!!
# Sonda je striktne interni a jeji kod nesmi byt verejne dostupny

try:
    import re
    import os
    import yaml
    import shlex
    import syslog
    import subprocess
except ImportError as err:
    print("<<<%s>>>" % "miner_detection")
    print("Import module error: '%s'" % err.message)
    exit(1)

default_config = {
	'safe_locations': [
		'/bin',
		'/sbin',
		'/usr',
		'/afs/ics.muni.cz/software',
	],
	'miner_process_params': [
		'stratum',
		'miner',
	],
	'miner_file_patterns' : [
        "stratum\+tcp",
		"stratum",
		"miner",
		"wallet",
		"coin",
	],
}

def check_params(command, user):
	for s in config['miner_process_params']:
		if s in command:
			print("A suspicious process found '%s' (user '%s')\n" % (command, user))
			return 1
	return 0

def check_executable(command, pid, exepath, user):
    found = False

    proc = subprocess.Popen(["strings", "-a", exepath] , stdout=subprocess.PIPE)

    pattern = re.compile('|'.join(config['miner_file_patterns']), flags=re.IGNORECASE)
    occurences = 0
    output = []
    while True:
        line = proc.stdout.readline().rstrip(os.linesep)
        if not line:
            break

        if pattern.search(line):
            occurences += 1
            output.append(line)

    if occurences > 1:
        found = True
        print("A suspicious process found '%s' (user '%s'): " % (command, user))
        print(", ".join(output))

    return found

# checkers return 1 if a suspicious process is found
def do_check(command, pid, exepath, user):
	if check_params(command, user):
		return 1

	if check_executable(command, pid, exepath, user):
		return 1

	return 0

def main():

	print("<<<%s>>>" % "miner_detection")

	detected = False

	procs = subprocess.check_output(shlex.split("/bin/ps auxwww --sort=-pcpu")).splitlines()[1:]
#   USER       PID %CPU %MEM    VSZ   RSS TTY      STAT START   TIME COMMAND
#   root         3  6.6  0.0      0     0 ?        S    Sep11 318:25 [ksoftirqd/0]

	for proc in procs:
		line = proc.split()
		user = line[0]
		pid = int(line[1])
		cpu = float(line[2])
		command = b" ".join(line[10:])

		if user == "root" or cpu < 30.0:
			continue

		exelink = "/proc/%d/exe" % pid
		try:
			exepath = os.readlink(exelink)
		except:
		    # probably a process that has finished in the meantime (kernel
			# processes are excluded above), should be harmless
			print("Unreadable executable found ('%s': %s), skipping\n" % (user, command))
			continue

		syslog.syslog("Miner check: processing command '%s' owned by user '%s'" % (command, user))

		fileinfo = exepath
		safe=False
		for i in config['safe_locations']:
			if fileinfo.startswith(i):
				safe = True
				break
		if safe:
			continue

		# By checking the /proc exlink we make sure we're not hit any permissions issues when
		# accessing files on remote file systems
		if do_check(command, pid, exelink, user) == 1:
			detected = True

	if not detected:
		print("No suspicious process detected\n")

	return detected


# basic initialization
config = default_config

if __name__ == "__main__":
	if main():
		exit(1)
	else:
	 	exit(0)
